/* ==========================================================================*/
/* - - - - - - - - - - -  B o o t M a i n . C   - - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*  Created   2007-06-29   msw                                               */
/*	Modified  2007-07-07   MW                                                */
/*.........................................................................  */
/* DV Dongle AT91SAM7256 Bootloader                                          */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "usart.h"
#include "timer.h"
#include "common.h"
#include "protocoldefs.h"
#include "AT91SAM7S256.h"
#define __inline static inline
#include "lib_AT91SAM7S256.h"


/*---------------------------------------------------------------------------*/
/*------------------------> M A C R O S <------------------------------------*/
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define FLASH_SECTOR_SIZE (FLASH_PAGE_LOCK*FLASH_PAGE_SIZE)
#define FLASH_SIZE (FLASH_PAGE_NB*FLASH_PAGE_SIZE)
#define CHKSUM_SIZE 4
#define CHKSUM_LOC (INT_FLASH_REMAP+FLASH_SECTOR_SIZE+0x3C)
	//last 4 bytes of exception irq vector table

#define FLASH_MODE_NORMAL (( (AT91C_MC_FMCN)&( ( (MCK/1000000)+1)<<16) ) | AT91C_MC_FWS_1FWS)
#define FLASH_MODE_ERASEPROG (( (AT91C_MC_FMCN)&( ( (MCK/666667)+1)<<16) ) | AT91C_MC_FWS_1FWS)

#define STATUS_BOOTIDLE 14		//status message state defines
#define STATUS_BOOTBUSY 15
#define STATUS_BOOTERASE 16
#define STATUS_BOOTERROR 0x80

#define MAX_MSG ( (256)+ 8 )	//256 bytes plus header
#define MAX_DATA (MAX_MSG-8)

#define MSG_TIMEOUT_TIME 45		//abt 2 second message timout

#define MSG_SIZE 0			// define some offsets into msg buffer
#define MSG_TYPE 1
#define MSG_CITEM 2
#define MSG_PARAM 4
#define MSG_DATAADR 2
#define MSG_DATA 6

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void SendResponseMsg(void);
static void ServiceHostMsg(U32BIT Length);
static void SetFlashLockBits(void);
static void ToggleLED(void);
static void WaitFRDY(void);
static int CalcCheckSum(void);
static int ProgramFlash( int* adr, int numbytes);

extern void _gotoapp(void);		//jumps into user code after copying vectors

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static BOOL UnLocked;
static BOOL GotMsg;
static BOOL SendResp;
static BOOL ProgOK;
static BOOL StayInBoot;
static U32BIT CurrentMsgPos;
static U32BIT MsgTimer;
static U32BIT SendMsgPos;
static AT91PS_MC pMC;

static U8BIT Msg[MAX_MSG];
static U8BIT Status;

const char ID_DESC[] = "DV Dongle Boot";

									// see CStartupBoot.S
extern U16BIT _VersionBoot;			// these are fixed FLASH memory locations
extern U16BIT _VersionHW;			// in the boot code
extern const char _SerialNumber[];

/* ==========================================================================*/
/*                                                                           */
/*                         MEMORY MAP                                        */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0020FFFF             */
/*      .          |                                 |                       */
/*      .          |              RAM                |                       */
/*      .          |                                 |                       */
/*     RAM         |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .--------->|---------------------------------|0x00200000             */
/*                                                                           */
/*                                                                           */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0013FFFF             */
/*      .          |                                 |                       */
/*      .          |    User  Flash Blocks           |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*      .          |                                 |                       */
/*    Flash        |                                 |                       */
/*      .          |.................................|0x00104040             */
/*      .          |   User Vectors and tables       |0x0010403F             */
/*      .          |   to be copied to RAM before    |                       */
/*      .          |  executing user code            |                       */
/*      .          |.................................|0x00104000             */
/*      .          |                                 |0x00103FFF             */
/*      .          | Protected BootLoader Block      |                       */
/*      .          |                                 |                       */
/*      .--------->|---------------------------------|0x00100000             */
/*                                                                           */
/*                                                                           */
/*      .--------> |---------------------------------|                       */
/*      .          |                                 |0x0003FFFF (0x0000FFFF)*/
/*      .          |                                 |           after remap)*/
/*      .          |    Flash or Ram after remap     |                       */
/*      .          |                                 |                       */
/*    Flash        |                                 |                       */
/*     or          |                                 |                       */
/*  (Ram after     |                                 |                       */
/*    remap)       |                                 |                       */
/*      .          |                                 |                       */
/*      .          |.................................|0x00000040             */
/*      .          | Vectors and jump tables         |0x0000003F             */
/*      .          | Copied from user flash before   |                       */
/*      .          |  jumping to user code           |                       */
/*      .--------->|---------------------------------|0x00000000             */
/* ==========================================================================*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------                                     */
/*                            | M a i n |                                    */
/*                             ---------                                     */
/*    This is the Boot Main program entry point.                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int main( void )
{
int ch;
int blinktimer;
U32BIT mLength = 0;
	AT91F_MC_Remap(); 	//jumps to RAM copy of bootloader code now mapped at 0x0

 	pMC = AT91C_BASE_MC;
    // enable the clock of the PIO
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;

	// configure the PIO Lines corresponding to LEDs to be output.
    // Turn Off the LEDs. A "1" turns off
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED1|LED2 ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, AMBE_RESET_ ) ;
	AT91F_PIO_CfgInput( AT91C_BASE_PIOA, TP1|LED2 ) ;
	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, LED1|LED2|AMBE_RESET_ ) ;
	pMC->MC_FMR = FLASH_MODE_NORMAL;

	UsartInit();
	TimerInit();
//	SetFlashLockBits();		//make sure boot is locked and rest of flash unlocked
	UnLocked = FALSE;
	ProgOK = FALSE;
	CurrentMsgPos = 0;
	GotMsg = FALSE;
	SendResp = FALSE;
	ProgOK = FALSE;
	UnLocked = FALSE;
	Status = STATUS_BOOTIDLE;
	MsgTimer = MSG_TIMEOUT_TIME;
	blinktimer = 1;
	ch = 0;
 	//emergency method to get to stay in boot mode if TP1 is held low after reset
 	//wait for port line to be settled
 	while(!(AT91C_BASE_TC0->TC_SR & AT91C_TC_COVFS) ){};
	while(!(AT91C_BASE_TC0->TC_SR & AT91C_TC_COVFS) ){};
	if( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & TP1) == 0 )
		StayInBoot = TRUE;
	else
		StayInBoot = FALSE;
	//if TP1 held low then stay in bootloader
    if( !StayInBoot )
    {
		//if entered from a software reset then stay in bootloader
		if( ( (*AT91C_RSTC_RSR) & AT91C_RSTC_RSTTYP) != AT91C_RSTC_RSTTYP_SOFTWARE)
	    {
			//if checksum error then stay in bootloader
//			if( CalcCheckSum() )
				_gotoapp();		//jump to user code if all ok
	    }
	}
//PrintDebug( "\n\r BootLoader Init", 1);
    // Main Loop
    for (;;)
    {

    	if( (ch = UsartService()) >= 0 )	//process usart characters as they come in
    	{	
			Msg[CurrentMsgPos++] = (U8BIT)ch;	//read a msg byte
			if(CurrentMsgPos >= MAX_MSG)
			{
				CurrentMsgPos--;	//clamp msg length to buffer size
			}
			if(CurrentMsgPos >= 2)
			{
				mLength = (U32BIT)Msg[MSG_SIZE+1];	//get ASCP message length
				mLength <<= 8; 
				mLength += (U32BIT)Msg[MSG_SIZE];
				mLength &= LENGTH_MASK;
				if( CurrentMsgPos >= mLength )
				{
					CurrentMsgPos = 0;	//got a complete msg
					GotMsg = TRUE;
				}
			}
     	}
 		if(GotMsg) // if a msg received from Host then parse it
 		{
			ServiceHostMsg(mLength);
 		}
		if(SendResp)	//if need to send response to host
			SendResponseMsg();
    	if(	AT91C_BASE_TC0->TC_SR & AT91C_TC_COVFS )	//if timer fired
    	{
			if(CurrentMsgPos != 0)	//if a msg is being received
			{
				if(--MsgTimer == 0 )	//if msg not fully received
				{						// then reset msg pointer
					CurrentMsgPos = 0;
					MsgTimer = MSG_TIMEOUT_TIME;
					Status = STATUS_BOOTIDLE;
					UnLocked = FALSE;
				}
			}
			else
			{	// reset message one shot timer
				MsgTimer = MSG_TIMEOUT_TIME;
			}
			blinktimer--;
			if(0 == blinktimer)	//blink LED
			{
				ToggleLED();
				if(!UnLocked)
					blinktimer = 10;
				else
					blinktimer = 2;
			}
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -----------------------------                          */
/*                   | S e r v i c e H o s t M s g |                         */
/*                    -----------------------------                          */
/*    This routine is called to parse a new message from the Host.           */
/* INPUTS: The Msg Buffer containing the new msg from the Host               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ServiceHostMsg(U32BIT Length)
{
U32BIT i;
U32BIT mType;
U32BIT mCitem;
t32BITS tmp;
U8BIT ch;
	mType = (U32BIT)(Msg[MSG_TYPE] & 0xE0);	//strip out just type field
	mCitem = (U32BIT)Msg[MSG_CITEM+1];
	mCitem <<= 8;
	mCitem |= (U32BIT)Msg[MSG_CITEM];
	if( mType == TYPE_HOST_SET_CITEM )
	{	// Is a message from host to set a control item value
		Msg[MSG_TYPE] = TYPE_TARG_RESP_CITEM;
		switch( mCitem )
		{
			case CI_UPDATE_MODE_CONTROL:
				if( (Msg[MSG_PARAM+2] == BOOTSECURE0 ) &&
					(Msg[MSG_PARAM+3] == BOOTSECURE1 ) &&
					(Msg[MSG_PARAM+4] == BOOTSECURE2 ) &&
					(Msg[MSG_PARAM+5] == BOOTSECURE3 ) )
				{	//is valid boot command
					if( Msg[MSG_PARAM+1] == CI_UPDATE_MODE_CONTROL_START )
					{
						Status = STATUS_BOOTBUSY;
						UnLocked = TRUE;	//allow programming blocks
						ProgOK = TRUE;
						pMC->MC_FMR = FLASH_MODE_ERASEPROG;	//set prog timing mode for programming
					}
					else if( Msg[MSG_PARAM+1] == CI_UPDATE_MODE_CONTROL_ENTER )
					{	//enter boot mode
							Status = STATUS_BOOTIDLE;
					}
					else if( (Msg[MSG_PARAM+1] == CI_UPDATE_MODE_CONTROL_END) && UnLocked && ProgOK)
					{	//is end of programming sequence
						UnLocked = FALSE;
						if( CalcCheckSum() )
						{
							Status = STATUS_BOOTIDLE;
							pMC->MC_FMR = FLASH_MODE_NORMAL;
							SendResponseMsg();	//ack here since are jumping to app code
							_gotoapp();			//jump to application
						}
						else
						{
							ProgOK = FALSE;
							Status = STATUS_BOOTERROR;
						}
					}
					else
					{
						ProgOK = FALSE;
						UnLocked = FALSE;
						Status = STATUS_BOOTERROR;
					}
				}
				else
				{
					Msg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK 
					Msg[MSG_TYPE] = 0; 
				}
				break;
			default:
				Msg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK 
				Msg[MSG_TYPE] = 0; 
				break;
		}
	}
	else if( mType == TYPE_HOST_REQ_CITEM )
	{	// Is a message from host requesting a control item value
		Msg[MSG_TYPE] = TYPE_TARG_RESP_CITEM;
		switch(mCitem)
		{	
			case CI_GENERAL_INTERFACE_NAME:
				i = 0;
				do{
					ch = ID_DESC[i];
					Msg[MSG_PARAM+i] = ch;
					i++;
				}while( (ch != 0) &&  (i<20) );
				Msg[MSG_SIZE] = 4+i;
				break;
			case CI_GENERAL_INTERFACE_VERSION:
				Msg[MSG_SIZE] = CI_GENERAL_INTERFACE_VERSION_SETRESPLEN;
				Msg[MSG_PARAM] = (ASCP_INTERFACE_VERSION&0x00FF);
				Msg[MSG_PARAM+1] = ((ASCP_INTERFACE_VERSION>>8)&0x00FF);
				break;
			case CI_GENERAL_INTERFACE_SERIALNUM:
				i = 0;
				do{
					ch = _SerialNumber[i];
					Msg[MSG_PARAM+i] = ch;
					i++;
				}while( (ch != 0) &&  (i<9) );
				Msg[MSG_SIZE] = 4+i;
				break;
			case CI_GENERAL_HARDFIRM_VERSION:
				Msg[MSG_SIZE] = CI_GENERAL_HARDFIRM_VERSION_SETRESPLEN;
				if(Msg[MSG_PARAM+0] == 0)	//boot code
				{
					Msg[MSG_PARAM+1] = _VersionBoot&0x00FF;
					Msg[MSG_PARAM+2] = (_VersionBoot>>8)&0x00FF;
				}
				else if(Msg[MSG_PARAM+0] == 1) //flash version
				{	
					// FLASH Program version not known in boot mode
					Msg[MSG_PARAM+1] =  0;
					Msg[MSG_PARAM+2] =  0;
				}
				else if(Msg[MSG_PARAM+0] == 2) 	//HW version
				{
					Msg[MSG_PARAM+1] = _VersionHW&0x00FF;
					Msg[MSG_PARAM+2] = (_VersionHW>>8)&0x00FF;
				}
				break;
			case CI_GENERAL_STATUS_CODE:
				Msg[MSG_SIZE] = CI_GENERAL_STATUS_CODE_SETRESPLEN;
				Msg[MSG_PARAM] = Status;
				break;
			case CI_UPDATE_MODE_PARAMS:
				Msg[MSG_SIZE] = CI_UPDATE_MODE_PARAMS_SETRESPLEN;
				tmp.all = FLASH_SIZE;
				Msg[MSG_PARAM+1] = tmp.bytes.b0;
				Msg[MSG_PARAM+2] = tmp.bytes.b1;
				Msg[MSG_PARAM+3] = tmp.bytes.b2;
				Msg[MSG_PARAM+4] = tmp.bytes.b3;
				tmp.all = FLASH_PAGE_SIZE;
				Msg[MSG_PARAM+5] = tmp.bytes.b0;
				Msg[MSG_PARAM+6] = tmp.bytes.b1;
				Msg[MSG_PARAM+7] = tmp.bytes.b2;
				Msg[MSG_PARAM+8] = tmp.bytes.b3;
				tmp.all = FLASH_SECTOR_SIZE;
				Msg[MSG_PARAM+9] = tmp.bytes.b0;
				Msg[MSG_PARAM+10] = tmp.bytes.b1;
				Msg[MSG_PARAM+11] = tmp.bytes.b2;
				Msg[MSG_PARAM+12] = tmp.bytes.b3;
				break;
			default:
				Msg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK 
				Msg[MSG_TYPE] = 0; 
				break;
		}
	}
	else if( mType == TYPE_HOST_DATA_ITEM0 )
	{	//got a block of data to program into the Flash Memory
		if(UnLocked && ProgOK)
		{
			tmp.bytes.b0 = Msg[MSG_DATAADR+0];
			tmp.bytes.b1 = Msg[MSG_DATAADR+1];
			tmp.bytes.b2 = Msg[MSG_DATAADR+2];
			tmp.bytes.b3 = Msg[MSG_DATAADR+3];
			if( (tmp.all <= BOOTFLASH_START) || (tmp.all <= BOOTFLASH_END) )
			{
				UnLocked = FALSE;
				ProgOK = FALSE;
				Status = STATUS_BOOTERROR;
			}
			else
			{
				if(	!ProgramFlash( (int*)tmp.all, Length-6 ) )
				{
					UnLocked = FALSE;
					ProgOK = FALSE;
					Status = STATUS_BOOTERROR;
				}
			}
		}
		if(ProgOK)
		{
			Msg[MSG_SIZE] = DATA_ITEM_ACK_LENGTH;
			Msg[MSG_TYPE] = TYPE_DATA_ITEM_ACK;
			Msg[MSG_DATAADR] = 0;	//ACK Data Item0
		}
		else
		{
			Status = STATUS_BOOTERROR;
			Msg[MSG_SIZE] = CI_GENERAL_STATUS_CODE_SETRESPLEN;
			Msg[MSG_TYPE] = TYPE_TARG_UNSOLICITED_CITEM; 
			Msg[MSG_CITEM] = CI_GENERAL_STATUS_CODE;
			Msg[MSG_CITEM+1] = 0;
			Msg[MSG_PARAM] = Status;
		}
	}
	else
	{
		Msg[MSG_SIZE] = TARG_RESP_NAK;	// Send NAK 
		Msg[MSG_TYPE] = 0; 
	}
	SendResp = TRUE;
	GotMsg = FALSE;
	SendMsgPos = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------------                       */
/*                    | S e n d R e s p o n s e M s g |                      */
/*                     -------------------------------                       */
/*    This routine is called to send a response to the Host.                 */
/* INPUTS: The Msg Buffer containing the msg to send back to the Host.       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void SendResponseMsg(void)
{
	while( SendMsgPos < Msg[MSG_SIZE] )	//Send resp buffer
		SendUart( Msg[SendMsgPos++] );
	SendMsgPos = 0;
	SendResp = FALSE;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   ---------------------------------                       */
/*                  | S e t F l a s h L o c k B i t s |                      */
/*                   ---------------------------------                       */
/*    Makes sure boot lock bit is set and rest are cleared.                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void SetFlashLockBits(void)
{
int adr;
int cmd;
int lockbits = AT91F_MC_EFC_GetStatus(pMC);
	lockbits &= 0xFFFF0000;
//PrintDebug( "\n\r Lockbits  =", lockbits);
	pMC->MC_FMR = FLASH_MODE_NORMAL;	//set timing mode for lock bit programming
	if(lockbits != 0x00010000)	//if need to change lockbit setting
	{
		cmd = AT91C_MC_FCMD_LOCK + 0x5A000000;	//lock boot in page 0
		WaitFRDY();
	 	AT91F_MC_EFC_PerformCmd( pMC, cmd);
		for(adr=FLASH_SECTOR_SIZE; adr<FLASH_SIZE; adr+=FLASH_SECTOR_SIZE)
		{
			cmd = adr/FLASH_PAGE_SIZE;
			cmd = (cmd<<8) + AT91C_MC_FCMD_UNLOCK + 0x5A000000;
			WaitFRDY();
		 	AT91F_MC_EFC_PerformCmd( pMC, cmd);
		}
		lockbits = AT91F_MC_EFC_GetStatus(pMC);
		lockbits &= 0xFFFF0000;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------------                               */
/*                  | P r o g r a m F l a s h |                              */
/*                   -------------------------                               */
/*    Programs one page of flash.                                            */
/*    numbytes must be same as FLASH_PAGE_SIZE								 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static int ProgramFlash( int* adr, int numbytes)
{
t32BITS tmp;
U32BIT* pRom;
int cmd;
int i;
int j;
	j = 0;
	pRom = (U32BIT*)INT_FLASH_REMAP;	//pointer to start of Flash memory
	for(i=0; i<FLASH_PAGE_SIZE; i+=4 )
	{
		tmp.bytes.b0 = Msg[MSG_DATA+i];
		tmp.bytes.b1 = Msg[MSG_DATA+i+1];
		tmp.bytes.b2 = Msg[MSG_DATA+i+2];
		tmp.bytes.b3 = Msg[MSG_DATA+i+3];
		pRom[j++] = tmp.all;
	}
	cmd = ( (int)adr & (FLASH_SIZE-1) )/FLASH_PAGE_SIZE;
	cmd = (cmd<<8) + AT91C_MC_FCMD_START_PROG + 0x5A000000;
//PrintDebug( "\n\rProgram - ", (U32BIT)adr);
	WaitFRDY();
	AT91F_MC_EFC_PerformCmd( pMC, cmd);
	WaitFRDY();
	return TRUE;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -----------------                                       */
/*                  | W a i t F R D Y |                                      */
/*                   -----------------                                       */
/*    Waits till Flash ready status bit is set.                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void WaitFRDY(void)
{
int status;
	do
	{
		status = AT91F_MC_EFC_GetStatus(pMC);
	}while( 0 == ( status & AT91C_MC_FRDY ) );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------                                     */
/*                  | T o g g l e L E D |                                    */
/*                   -------------------                                     */
/*    Toggle LED1 and 2.                                                     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ToggleLED(void)
{
	if ( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & LED1 ) == LED1 )
	{
    	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED1 );
	    AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 );
 	}
	else
	{
	    AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED1 );
    	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED2 );
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------------                               */
/*                  | C a l c C h e c k S u m |                              */
/*                   -------------------------                               */
/*    Returns TRUE if application flash has valid checksum.                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static int CalcCheckSum(void)
{
U32BIT chksum;
U32BIT* pChkLoc;
U32BIT* pChk;
U8BIT* adr;
	pChkLoc = (U32BIT*)CHKSUM_LOC;
	if( (*pChkLoc >= FLASH_SIZE )||	//make sure reasonable size in case of blank
		( *pChkLoc < 0x40 ) )		// flash
	{
		return 0;
	}
	chksum = 0;
	for( adr=(U8BIT*)(INT_FLASH_REMAP+FLASH_SECTOR_SIZE);
			adr<(U8BIT*)(INT_FLASH_REMAP+*pChkLoc); adr++ )
	{
		chksum += *adr;
	}
	pChk = (U32BIT*)(INT_FLASH_REMAP + *pChkLoc);
	if( *pChk == chksum )
		return 1;
	else
		return 0;
}

