/*------------------------------------------------------------------------------
//*                   C S t a r t u p B o o t . S
//*------------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*-----------------------------------------------------------------------------
//*- File source          : CstartupBoot.s
//*- Object               : CStartup for GCC AT91SAM7256 Bootloader code (Uses REMAP)
//*- Compilation flag     : None
//*-
//*- 1.0 2005/01/29 MSW   : Creation
//*-----------------------------------------------------------------------------*/
	/* Stack Sizes (no interrupts are used) */
	.set  UND_STACK_SIZE, 0x00000004
	.set  ABT_STACK_SIZE, 0x00000004
	.set  FIQ_STACK_SIZE, 0x00000004
	.set  IRQ_STACK_SIZE, 0X00000004
	.set  SVC_STACK_SIZE, 0x00000004

	.set AIC_IVR,         (256) 
	.set AIC_FVR,         (260) 
	.set AIC_EOICR,       (304)
	.set AT91C_BASE_AIC,  (0xFFFFF000)
	.set SBOOT_RAM,	(0x00200000)	/*Beginning of Boot RAM area */
	.set EBOOT_RAM,	(0x00204000)	/*End of Boot RAM area */
	
	.set SVECT_RAM,	(0x00000000)	/*Start of vector table in RAM area */
	.set EVECT_RAM,	(0x00000040)	/*End of vector table in RAM area */
	.set SVECT_FLASH,(0x00104000)	/*Start of vector table in user flash area */

#;------------------------------------------------------------------------------
#;- Section Definition
#;-----------------
#;-  Section    
#;- .internal_ram_top 	Top_Stack: used by the cstartup for vector initalization 
#;-						management defined by ld and affect from ldscript 
#;------------------------------------------------------------------------------
	.section 	.internal_ram_top
	.code 32
	.align 	0
	.global	Top_Stack
Top_Stack:
	
/*------------------------------------------------------------------------------
*- Code Area Definition
*------------------------------------------------------------------------------*/
         .section .startup
        .global _startup
		.global	_VersionBoot
		.global	_VersionHW
		.global	_SerialNumber
        .func   _startup
_startup:
reset: 		/* 0x00000000 */
/*------------------------------------------------------------------------------*/
/*- Exception vectors (IRQs unused in boot code)                                */
/*------------------------------------------------------------------------------*/
		B           InitReset           /* 0x00 Reset handler */
undefvec:
		B           undefvec            /* 0x04 Undefined Instruction */
swivec:
		B           swivec              /* 0x08 Software Interrupt */
pabtvec:
		B           pabtvec             /* 0x0C Prefetch Abort */
dabtvec:
		B           dabtvec             /* 0x10 Data Abort */
rsvdvec:
		B           rsvdvec             /* 0x14 reserved  */
irqvec:
		B           irqvec   			/* 0x18 IRQ	 */
fiqvec:               			            	
		B           fiqvec				/* 0x1c FIQ	*/

/*------------------------------------------------------------------------------*/
/*  Shared Constants with Application Code                                      */
/*------------------------------------------------------------------------------*/
_Filler:	.skip 0x20 		/* filler                   */
_VersionBoot:	.hword 104 		/* 0x00100040 location of Boot code Version */
_VersionHW:		.hword 200 		/* 0x00100042 location of HW Version        */
_SerialNumber:	.asciz "DV000007" 	/* 0x00100044 location of 8 byte Serial Num string */
									/* this gets overwritten in production programmer */

	.align 0
.RAM_TOP:
	.word	Top_Stack

InitReset:
/*------------------------------------------------------------------------------
/*- Low level Init (PMC, AIC, ....) by C function AT91F_LowLevelInit
/*------------------------------------------------------------------------------*/
	.extern   AT91F_LowLevelInit
/*- minumum C initialization */
/*- calls  AT91F_LowLevelInit( void) */

	ldr     r13,.RAM_TOP 	/* temporary stack in internal RAM */
/*--Call Low level init function in ABSOLUTE through the Interworking	*/
	ldr	    r0,=AT91F_LowLevelInit
	mov     lr, pc
	bx	    r0

/*------------------------------------------------------------------------------
//*- ARM_MODE_SVC (Application, C) Stack is located at the top of the external memory.
//*------------------------------------------------------------------------------*/

	.set	IRQ_STACK_SIZE, (3*8*4)
	.set	ARM_MODE_USR, 0x10	/* User Mode */
	.set	ARM_MODE_FIQ, 0x11	/* FIQ Mode */
	.set	ARM_MODE_IRQ, 0x12	/* IRQ Mode */
	.set	ARM_MODE_SVC, 0x13	/* Supervisor Mode */
	.set	ARM_MODE_ABT, 0x17	/* Abort Mode */
	.set	ARM_MODE_UND, 0x1B	/* Undefined Mode */
	.set	ARM_MODE_SYS, 0x1F	/* System Mode */
	.set	I_BIT, 0x80
	.set	F_BIT, 0x40

/*------------------------------------------------------------------------------
//*- Setup the stack for each mode
//*-------------------------------*/
	mov   r0,sp
	msr   CPSR_c, #ARM_MODE_UND|I_BIT|F_BIT 	/* Undefined Instruction Mode  */
    mov   sp, r0
    sub   r0, r0, #UND_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_ABT|I_BIT|F_BIT 	/* Abort Mode */
    mov   sp, r0
    sub   r0, r0, #ABT_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_FIQ|I_BIT|F_BIT 	/* FIQ Mode */
   	ldr   r8, =AT91C_BASE_AIC
    mov   sp, r0	
   	sub   r0, r0, #FIQ_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_IRQ|I_BIT|F_BIT 	/* IRQ Mode */
    mov   sp, r0
    sub   r0, r0, #IRQ_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_SVC|I_BIT|F_BIT 	/* Supervisor Mode */
    mov   sp, r0
    sub   r0, r0, #SVC_STACK_SIZE
    msr   CPSR_c, #ARM_MODE_SYS|I_BIT|F_BIT 	/* execute now in User Mode */
    mov   sp, r0								/* with stack at memory top */

# Relocate .data section (Copy from ROM to RAM)
                LDR     R1, =_etext
                LDR     R2, =_data
                LDR     R3, =_edata
LoopRel:        CMP     R2, R3
                LDRLO   R0, [R1], #4
                STRLO   R0, [R2], #4
                BLO     LoopRel

# Clear .bss section (Zero init RAM)
                MOV     R0, #0
                LDR     R1, =_bss_start
                LDR     R2, =_bss_end
LoopZI:         CMP     R1, R2
                STRLO   R0, [R1], #4
                BLO     LoopZI

# Relocate .text section (Copy boot code from ROM to RAM)
                LDR     R1, =_text
                LDR     R2, =SBOOT_RAM
                LDR     R3, =EBOOT_RAM
LoopBoot:       CMP     R2, R3
                LDRLO   R0, [R1], #4
                STRLO   R0, [R2], #4
                BLO     LoopBoot


				ldr	lr,=exit		/*Call main() where REMAP will take place */
				ldr	r0,=main		/* so bootloader will run out of ram */
				bx	r0

		        .size   _startup, . - _startup
        .endfunc
/* --------------------------------------------------------------------------*/
/* called by boot code to jump from boot code into user downloaded flash code*/
/*  Copies user exception vector table to RAM at location 0x0 then jumps to  */
/*  application code startup.  Memory remap has already occured.             */
/* --------------------------------------------------------------------------*/
        .global _gotoapp
        .func   _gotoapp
_gotoapp:
		LDR     R1, =SVECT_FLASH	/* source start address */
		LDR     R2, =SVECT_RAM		/* destination start address */
		LDR     R3, =EVECT_RAM		/* destination end address */
Loopcpy:
		CMP     R2, R3
		LDRLO   R0, [R1], #4
		STRLO   R0, [R2], #4
		BLO     Loopcpy
		LDR		PC, =0				/*jump to new application reset vector */
        .endfunc

/* "exit" dummy added by mthomas to avoid sbrk write read etc. needed
   by the newlib default "exit" */
        .global exit
        .func   exit
exit:
        b    .
	.size   exit, . - exit
        .endfunc

        .end

	
