/* ==========================================================================*/
/* - - - - - - - - - - - - - -  U s a r t . C   - - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*	Created   2005-11-03   M. Wheatley                                       */
/*	Modified  2006-11-16   MW                                                */
/*.........................................................................  */
/* DV Dongle AT91SAM7256 Bootloader                                          */
//////////////////////////////////////////////////////////////////////////////
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "usart.h"
#include "AT91SAM7S256.h"
#define __inline static inline
#include "lib_AT91SAM7S256.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define ASYNC_MODE ( AT91C_US_USMODE_NORMAL + \
                        AT91C_US_NBSTOP_1_BIT + \
                        AT91C_US_PAR_NONE + \
                        AT91C_US_CHRL_8_BITS + \
                        AT91C_US_CLKS_CLOCK )

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   --------------------------                              */
/*                  |  U s a r t S e r v i c e |                             */
/*                   --------------------------                              */
/*    Called by Main loop to check for and get any received data.            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int UsartService(void)
{
unsigned int status;
int ret;
	ret = -1;
	//* get debug Usart status register
	status = *AT91C_US0_CSR;
	if ( status & AT91C_US_RXRDY)
	{
		//* Get byte
		ret = *AT91C_US0_RHR;
	}

	if ( status & (AT91C_US_OVRE|AT91C_US_FRAME) )
	{
		//* clear US_RXRDY
		ret = *AT91C_US0_RHR;
		ret = -1;
	}

	//* Reset the status bit
	*AT91C_US0_CR = AT91C_US_RSTSTA;
	 return ret;
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------                                      */
/*                 | U s a r t I n i t |                                     */
/*                  -------------------                                      */
/*    Called to initialize the Usart.                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void UsartInit( void )
{
 	//* Configure PIO controllers to periph mode
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		((unsigned int) RXD    ) |
 		((unsigned int) TXD    ),// Peripheral A
 		0); // Peripheral B
   	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_US0 ) ;
	// Usart Configure
 	AT91C_BASE_US0->US_BRGR = MCK/(16*UART_BAUD);
 	AT91C_BASE_US0->US_MR = ASYNC_MODE;
	AT91C_BASE_US0->US_TTGR = 0;
	// Enable usart
	AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;

    //* Enable USART IT error and RXRDY
	AT91F_US_EnableIt(AT91C_BASE_US0,AT91C_US_TIMEOUT | AT91C_US_FRAME | AT91C_US_OVRE |AT91C_US_RXRDY);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | P r i n t D e b u g |                               */
/*                      ---------------------                                */
/*    Called to send a string and 32bit value out the usart for debugging.   */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void PrintDebug(char* str, unsigned int val)
{
int i;
unsigned int x;
unsigned char ch;
	i = 0;
	while( 0 != (ch=str[i]) )
	{
		SendUart(ch);
		i++;
	}
	SendUart('x');
	for(i=0; i<8; i++)
	{
		x = (unsigned int)(val&0xF0000000);
		x >>= 28;
		ch = (char)x;
		if(ch<10)
		{
			ch += '0';
		}
		else
		{
			ch += ('A'-10);
		}
		SendUart(ch);
		if(i==3)
			SendUart(' ');
		val = val<<4;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------                                    */
/*                     | S e n d U a r t |                                   */
/*                      -----------------                                    */
/* Called to send one byte out the usart. (waits for previous byte to finish)*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendUart(unsigned char ch)
{
unsigned int status;
	do
	{
		status = *AT91C_US0_CSR;
	}while( 0 == (status & AT91C_US_TXRDY) );
	*AT91C_US0_THR = ch;
}
